/*
 * Copyright (c) 2022 Lucas Dietrich <ld.adecy@gmail.com>
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#ifndef _IDLE_H_
#define _IDLE_H_

#include <stdbool.h>
#include <stddef.h>

#include "kernel.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Idle thread structure
 */
extern struct k_thread z_thread_idle;

/**
 * @brief Tells whether the runqueue contains the thread IDLE
 *
 * @return true
 * @return false
 */

__kernel bool k_is_cpu_idle(void);

/**
 * @brief IDLE the CPU.
 *
 * This function can be called from a thread to make it behave as an IDLE
 * thread.
 *
 * Important: if others thread a ready, they will be blocked until an interrupt
 * occurs !
 *
 * Note: This function requires interrupts to be enabled.
 */
__kernel void k_idle(void);

#ifdef __cplusplus
}
#endif

#endif